# -*- coding: utf-8 -*-
#
# Copyright (c) 2019 Jordi Mas i Hernandez <jmas@softcatala.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.


import unittest
import tempfile
from store.session import Session
from store.xmlsessionstore import XMLSessionStore
from os import path

class TestXmlSessionStore(unittest.TestCase):

    def test_load(self):

        store_file = path.dirname(path.realpath(__file__))
        store_file += '/test-saved-sessions.xml'
        print(store_file)

        store = XMLSessionStore(store_file)
        self.assertEqual(3, len(store))
        self.assertEqual('files', store[0].name)
        self.assertEqual('gspell', store[1].name)
        self.assertEqual('window-activable', store[2].name)

        self.assertEqual(1, len(store[0].files))
        self.assertEqual("/xmlsessionstore.py", store[0].files[0].get_path())

    def test_save(self):
        session_name = "session_A"
        tmpfile = tempfile.NamedTemporaryFile()
        store = XMLSessionStore(tmpfile.name)
        session = Session(session_name)
        store.add(session)
        store.save()

        store = XMLSessionStore(tmpfile.name)
        self.assertEqual(1, len(store))
        self.assertEqual(session_name, store[0].name)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2017 Jordi Mas i Hernandez <jmas@softcatala.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.


import unittest
from services.yandex import Yandex
from unittest.mock import patch, MagicMock


class TestYandex(unittest.TestCase):

    @patch('urllib.request.urlopen')
    def test_translate_text(self, mock_urlopen):
        cm = MagicMock()
        cm.getcode.return_value = 200
        cm.read.return_value = bytes('{"code":200,"lang":"ca-en","text":["Hello friends"]}', 'utf-8')
        cm.__enter__.return_value = cm
        mock_urlopen.return_value = cm

        yandex = Yandex()
        yandex.set_api_key('our_key')
        translated = yandex.translate_text('Hola amics', 'ca|en')
        self.assertEqual('Hello friends', translated)
        url = "https://translate.yandex.net/api/v1.5/tr.json/translate?lang=ca-en&format=plain&key=our_key&text=Hola+amics"
        mock_urlopen.assert_called_with(url)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
#
#  Copyrignt (C) 2017 Jordi Mas <jmas@softcatala.org>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor,
#  Boston, MA 02110-1301, USA.

from .apertium import Apertium
from .yandex import Yandex


class Services():

    APERTIUM_ID = 0
    YANDEX_ID = 1
    SERVICES = {APERTIUM_ID: "Apertium", YANDEX_ID: "Yandex"}

    @staticmethod
    def get_name(service_id):
        return Services.SERVICES[service_id]

    @staticmethod
    def get(service_id):
        if service_id == Services.APERTIUM_ID:
            return Apertium()
        elif service_id == Services.YANDEX_ID:
            return Yandex()

    @staticmethod
    def get_names_and_ids():
        return Services.SERVICES



# -*- coding: utf-8 -*-
#
#  grep.py - grep commander module
#
#  Copyright (C) 2010 - Jesse van den Kieboom
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor,
#  Boston, MA 02110-1301, USA.

import commander.commands as commands
import commander.commands.completion
import commander.commands.result
import commander.commands.exceptions

import re

__commander_module__ = True
HideTagName = 'CommanderModuleGrepHideTag'
ZoomOutTagName = 'CommanderModuleGrepZoomOutTag'
HighlightTagName = 'CommanderModuleGrepHighlightTag'

def _get_tag(buf, name, callback=None, **args):
    table = buf.get_tag_table()
    tag = table.lookup(name)

    if tag is None:
        if not callback is None:
            args.update(callback(buf))

        tag = buf.create_tag(name, **args)

    return tag

def _get_invisible_tag(buf):
    return _get_tag(buf, HideTagName, invisible=True, invisible_set=True)

def _get_zoomout_tag(buf):
    return _get_tag(buf, ZoomOutTagName, size_points=4)

def _create_highlight_tag(buf):
    scheme = buf.get_style_scheme()
    style = scheme.get_style('search-match')

    if style is None:
        return {'background': '#FFFF78', 'background_set': True, 'foreground_set': False}

    ret = {}

    if style.props.foreground_set:
        ret.update({'foreground_set': True, 'foreground': style.props.foreground})

    if style.props.background_set:
        ret.update({'background_set': True, 'background': style.props.background})

    return ret

def _get_highlight_tag(buf):
    return _get_tag(buf, HighlightTagName, _create_highlight_tag)

def _grep_action_hide(view, start, end):
    # Apply tag that makes line invisible
    buf = view.get_buffer()
    tag = _get_invisible_tag(buf)

    buf.apply_tag(tag, start, end)

def _grep_action_show(view, start, end):
    buf = view.get_buffer()
    tag = _get_invisible_tag(buf)

    buf.remove_tag(tag, start, end)

def _grep_action_zoomin(view, start, end):
    buf = view.get_buffer()
    tag = _get_zoomout_tag(buf)

    buf.remove_tag(tag, start, end)

def _grep_action_zoomout(view, start, end):
    buf = view.get_buffer()
    tag = _get_zoomout_tag(buf)

    buf.apply_tag(tag, start, end)

def _highlight(view, start, end, matches):
    buf = view.get_buffer()
    tag = _get_highlight_tag(buf)

    for match in matches:
        st = start.copy()
        ed = start.copy()

        st.forward_chars(match.start(0))
        ed.forward_chars(match.end(0))

        buf.apply_tag(tag, st, ed)

def _unhighlight(view, start, end, matches):
    buf = view.get_buffer()
    tag = _get_highlight_tag(buf)

    for match in matches:
        st = start.copy()
        ed = start.copy()

        st.forward_chars(match.start(0))
        ed.forward_chars(match.end(0))

        buf.remove_tag(tag, st, ed)

def _grep(view, regex, match_action, non_match_action):
    buf = view.get_buffer()
    start = buf.get_start_iter()

    try:
        reg = re.compile(regex)
    except Exception as e:
        raise commands.exceptions.Execute('Invalid regular expression: ' + str(e))

    while True:
        end = start.copy()

        if not end.forward_line():
            end.forward_to_line_end()

        text = start.get_text(end)
        piter = list(reg.finditer(text))

        if not piter:
            _unhighlight(view, start, end, piter)
            non_match_action(view, start, end)
        else:
            _highlight(view, start, end, piter)
            match_action(view, start, end)

        if not start.forward_line():
            break

def __default__(view, argstr):
    """Hide non-matching lines in document: grep &lt;regex&gt;

Matches a regular expression on each line and hides all text that does not
match. For the revere (hiding matches) use grep.hide"""
    yield _grep(view, argstr, _grep_action_show, _grep_action_hide)

def hide(view, argstr):
    """Hide matching lines in document: grep.hide &lt;regex&gt;

Matches a regular expression on each line and hides all matches. For
the reverse (hiding lines that do not match) use grep.show"""
    yield _grep(view, argstr, _grep_action_hide, _grep_action_show)

def zoomin(view, argstr):
    """Zoom in on matching lines in document: grep.zoomin &lt;regex&gt;

Matches a regular expression on each line and magnifies all matching lines
with respect to the non-matching lines. For the reverse, use grep.zoomout"""
    yield _grep(view, argstr, _grep_action_zoomin, _grep_action_zoomout)

def zoomout(view, argstr):
    """Zoom out on matching lines in document: grep.zoomout &lt;regex&gt;

Matches a regular expression on each line and minifies all matching lines
with respect to the non-matching lines. For the reverse, use grep.zoomin"""
    yield _grep(view, argstr, _grep_action_zoomout, _grep_action_zoomin)

def clear(view):
    """Clear all grep commands: grep.clear

Clear the actions done by all grep commands."""
    buf = view.get_buffer()

    buf.remove_tag(_get_highlight_tag(buf), buf.get_start_iter(), buf.get_end_iter())
    buf.remove_tag(_get_invisible_tag(buf), buf.get_start_iter(), buf.get_end_iter())
    buf.remove_tag(_get_zoomout_tag(buf), buf.get_start_iter(), buf.get_end_iter())

locals()['show'] = __default__
locals()['zoom'] = zoomin

# vi:ts=4:et



# -*- coding: utf-8 -*-
#
#  shell.py - shell commander module
#
#  Copyright (C) 2010 - Jesse van den Kieboom
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor,
#  Boston, MA 02110-1301, USA.

import subprocess
import fcntl
import os
import tempfile
import signal
from gi.repository import GLib, GObject, Gio

import commander.commands as commands
import commander.commands.exceptions
import commander.commands.result

__commander_module__ = True
__root__ = ['!', '!!', '!&']

class Process:
    def __init__(self, entry, pipe, replace, background, tmpin, stdout, suspend):
        self.pipe = pipe
        self.replace = replace
        self.tmpin = tmpin
        self.entry = entry
        self.suspend = suspend

        if replace:
            self.entry.view().set_editable(False)

        if not background:
            fcntl.fcntl(stdout, fcntl.F_SETFL, os.O_NONBLOCK)
            conditions = GLib.IOCondition.IN | GLib.IOCondition.PRI | GLib.IOCondition.ERR | GLib.IOCondition.HUP

            channel = GLib.IOChannel.unix_new(stdout.fileno())
            self.watch = GLib.io_add_watch(channel, GLib.PRIORITY_DEFAULT, conditions, self.collect_output)
            self._buffer = ''
        else:
            stdout.close()

    def update(self):
        parts = self._buffer.split("\n")

        for p in parts[:-1]:
            self.entry.info_show(p)

        self._buffer = parts[-1]

    def collect_output(self, fd, condition):
        if condition & (GLib.IOCondition.IN | GLib.IOCondition.PRI):
            try:
                ret = fd.read()

                # This seems to happen on OS X...
                if len(ret) == 0:
                    condition = condition | GLib.IOCondition.HUP
                else:
                    self._buffer += ret.decode('utf-8')

                    if not self.replace:
                        self.update()
            except Exception as e:
                if len(self._buffer) > 0:
                    self.entry.info_show(self._buffer.strip("\n"))

                self.stop()
                return False

        if condition & (GLib.IOCondition.ERR | GLib.IOCondition.HUP):
            if self.replace:
                buf = self.entry.view().get_buffer()
                buf.begin_user_action()

                bounds = buf.get_selection_bounds()

                if bounds:
                    buf.delete(bounds[0], bounds[1])

                buf.insert_at_cursor(self._buffer)
                buf.end_user_action()
            elif len(self._buffer) > 0:
                self.entry.info_show(self._buffer.strip("\n"))

            self.stop()
            return False

        return True

    def stop(self):
        if not self.suspend:
            return

        if hasattr(self.pipe, 'kill'):
            self.pipe.kill()

        GLib.source_remove(self.watch)

        if self.replace:
            self.entry.view().set_editable(True)

        if self.tmpin:
            self.tmpin.close()

        sus = self.suspend
        self.suspend = None
        sus.resume()

def _run_command(entry, replace, background, argstr):
    tmpin = None

    cwd = None
    doc = entry.view().get_buffer()

    if not doc.is_untitled() and doc.get_file().is_local():
        gfile = doc.get_file().get_location()
        cwd = os.path.dirname(gfile.get_path())

    if '<!' in argstr:
        bounds = entry.view().get_buffer().get_selection_bounds()

        if not bounds:
            bounds = entry.view().get_buffer().get_bounds()

        inp = bounds[0].get_text(bounds[1])

        # Write to temporary file
        tmpin = tempfile.NamedTemporaryFile(delete=False)
        tmpin.write(inp.encode('utf-8'))
        tmpin.flush()

        # Replace with temporary file
        argstr = argstr.replace('<!', '< "' + tmpin.name + '"')

    try:
        p = subprocess.Popen(argstr, shell=True, cwd=cwd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
        stdout = p.stdout

    except Exception as e:
        raise commander.commands.exceptions.Execute('Failed to execute: ' + e)

    suspend = None

    if not background:
        suspend = commander.commands.result.Suspend()

    proc = Process(entry, p, replace, background, tmpin, stdout, suspend)

    if not background:
        yield suspend

        # Cancelled or simply done
        proc.stop()

        yield commander.commands.result.DONE
    else:
        yield commander.commands.result.HIDE

def __default__(entry, argstr):
    """Run shell command: ! &lt;command&gt;

You can use <b>&lt;!</b> as a special input meaning the current selection or current
document."""
    return _run_command(entry, False, False, argstr)

def background(entry, argstr):
    """Run shell command in the background: !&amp; &lt;command&gt;

You can use <b>&lt;!</b> as a special input meaning the current selection or current
document."""
    return _run_command(entry, False, True, argstr)

def replace(entry, argstr):
    """Run shell command and place output in document: !! &lt;command&gt;

You can use <b>&lt;!</b> as a special input meaning the current selection or current
document."""
    return _run_command(entry, True, False, argstr)

locals()['!'] = __default__
locals()['!!'] = replace
locals()['!&'] = background

# ex:ts=4:et



# -*- coding: utf-8 -*-
#
#  Copyrignt (C) 2017 Jordi Mas <jmas@softcatala.org>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor,
#  Boston, MA 02110-1301, USA.

import os
from gi.repository import Gio, Gtk
from gpdefs import GETTEXT_PACKAGE
from .services.services import Services
from .settings import Settings

try:
    import gettext
    gettext.bindtextdomain('gedit-plugins')
    gettext.textdomain('gedit-plugins')
    _ = gettext.gettext
except:
    _ = lambda s: s

class Preferences(object):

    LANG_NAME = 0
    LANG_CODE = 1

    def __init__(self, datadir, get_languages_names_codes):
        self._get_languages_names_codes = get_languages_names_codes
        self._settings = Settings()
        self._service_id = self._settings.get_service()

        self._load_ui(datadir)
        self._init_radiobuttons()
        self._init_combobox_languages()
        self._init_combobox_services()
        self._init_api_entry()

    def _load_ui(self, datadir):
        self._ui_path = os.path.join(datadir, 'ui', 'preferences.ui')
        self._ui = Gtk.Builder()
        self._ui.set_translation_domain(GETTEXT_PACKAGE)
        self._ui.add_from_file(self._ui_path)

    def _get_default_language(self):
        service = Services.get(self._service_id)
        selected = service.get_default_language_codes()
        return self._get_index(selected)

    def _populate_languages(self):
        self._languages.set_wrap_width(3)
        self._language_names, self._language_codes = self._get_languages_names_codes(self._service_id)
        self._model = self._get_languages_stored_model()
        self._languages.set_model(self._model)
        selected = self._settings.get_language_pair()
        index = self._get_index(selected)
        if index == -1:
            index = self._get_default_language()

        self._languages.set_active(index)

    def _init_api_entry(self):
        service = Services.get(self._service_id)
        if service.has_api_key() is True:
            self._update_api_key_ui(True)

    def _update_api_key_ui(self, show):
        apibox_container = self._ui.get_object('api_box_container')
        apibox = self._ui.get_object('api_box')

        if show is True:
            service = Services.get(self._service_id)
            self._apilabel = Gtk.Label(_("API Key"))
            self._apikey = Gtk.Entry(expand=True)
            self._apikey.connect('changed', self._changed_apikey)
            key = self._settings.get_apikey()
            self._apikey.set_text(key)

            apibox.add(self._apilabel)
            apibox.add(self._apikey)

            self._hint = Gtk.Label()
            self._hint.set_markup(service.get_api_hint())
            self._apilabel.set_margin_right(12)
            apibox_container.add(self._hint)
            apibox_container.show_all()
        else:
            apibox.remove(self._apilabel)
            apibox.remove(self._apikey)
            apibox_container.remove(self._hint)
            self._apilabel = None
            self._apikey = None
            self._hint = None

    def _init_radiobuttons(self):
        self._radio_samedoc = self._ui.get_object('same_document')
        self._output_window = self._ui.get_object('output_window')
        self._radio_samedoc.connect("toggled", self._radio_samedoc_callback)
        active = self._settings.get_output_document()

        if active:
            self._radio_samedoc.set_active(active)
        else:
            self._output_window.set_active(active is False)

    def _init_combobox_services(self):
        self._services = self._ui.get_object('services')

        cell = Gtk.CellRendererText()
        self._services.pack_start(cell, 1)
        self._services.add_attribute(cell, 'text', 0)
        services = Services.get_names_and_ids()

        model = Gtk.ListStore(str, int)
        for service_id in services.keys():
            model.append((services[service_id], service_id))

        self._services.set_model(model)
        service_id = self._settings.get_service()
        self._services.set_active(service_id)
        self._services.connect('changed', self._changed_services)

    def _init_combobox_languages(self):
        self._languages = self._ui.get_object('languages')

        cell = Gtk.CellRendererText()
        self._languages.pack_start(cell, 1)
        self._languages.add_attribute(cell, 'text', 0)
        self._populate_languages()
        self._languages.connect('changed', self._changed_lang_pair)

    def _get_languages_stored_model(self):
        sorted_language_names = set()

        for i in range(len(self._language_names)):
            sorted_language_names.add((self._language_names[i], self._language_codes[i]))

        sorted_language_names = sorted(sorted_language_names,
                                       key=lambda tup: tup[Preferences.LANG_NAME])

        model = Gtk.ListStore(str, str)
        for name_code in sorted_language_names:
            model.append(name_code)

        return model

    def _get_index(self, selected):
        for i in range(len(self._model)):
            if self._model[i][Preferences.LANG_CODE] == selected:
                return i
        return -1

    def _changed_lang_pair(self, combobox):
        model = combobox.get_model()
        index = combobox.get_active()
        if index > -1:
            item = model[index]
            self._settings.set_language_pair(item[Preferences.LANG_CODE])

    def _changed_services(self, combobox):
        model = combobox.get_model()
        index = combobox.get_active()
        if index == -1:
            return

        item = model[index]
        self._service_id = item[1]
        service = Services.get(self._service_id)
        if service.has_api_key() is True:
            key = self._settings.get_apikey()
            service.set_api_key(key)
        else:
            self._settings.set_service(self._service_id)

        service.init()
        self._update_api_key_ui(service.has_api_key())
        self._populate_languages()

    def _changed_apikey(self, text_entry):
        text = text_entry.get_text()
        self._settings.set_apikey(text)
        if len(text) > 0:
            self._settings.set_service(self._service_id)
        else:
            self._settings.set_service(Services.APERTIUM_ID)


    def _radio_samedoc_callback(self, widget, data=None):
        self._settings.set_output_document(widget.get_active())

    def configure_widget(self):
        self._ui.connect_signals(self)
        widget = self._ui.get_object('grid')
        return widget



# -*- coding: utf-8 -*-
#
# Copyright (c) 2017 Jordi Mas i Hernandez <jmas@softcatala.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.


import unittest
from services.apertium import Apertium
from unittest.mock import patch, MagicMock
from unittest.mock import Mock


class TestApertium(unittest.TestCase):

    def setUp(self):
        Apertium._clean_for_ut()

    def _get_apertium(self):
        apertium = Apertium()
        apertium.set_server("https://www.apertium.org/apy")
        return apertium

    @patch('urllib.request.urlopen')
    def test_translate_text(self, mock_urlopen):
        cm = MagicMock()
        cm.getcode.return_value = 200
        cm.read.return_value = bytes('{"responseData": {"translatedText": "Hauries d\'haver-hi rebut una c\u00f2pia"}, "responseDetails": null, "responseStatus": 200}', 'utf-8')
        cm.__enter__.return_value = cm
        mock_urlopen.return_value = cm

        apertium = self._get_apertium()
        translated = apertium.translate_text('You should have received a copy', 'eng|cat')
        self.assertEqual('Hauries d\'haver-hi rebut una còpia', translated)
        mock_urlopen.assert_called_with("https://www.apertium.org/apy/translate?langpair=eng|cat&markUnknown=no&q=You+should+have+received+a+copy")

    def test_fetch_remote_language_names_and_pairs_localized(self):
        mockObject = self._get_apertium()
        mockObject._get_user_locale = Mock(return_value='ca')
        mockObject._get_remote_language_pairs = Mock(return_value=[['es'], ['en'], ['es', 'en'], ['es|en']])
        mockObject._get_remote_language_names = Mock(return_value={'es': 'Espanyol', 'en': 'Anglès'})

        mockObject._fetch_remote_language_names_and_pairs()
        self.assertEqual({'en': 'Anglès', 'es': 'Espanyol'}, Apertium.g_locales_names)
        self.assertEqual(['Espanyol -> Anglès'], Apertium.g_language_names)
        self.assertEqual(['es|en'], Apertium.g_language_codes)

    def test_fetch_remote_language_names_and_pairs_non_localized(self):
        mockObject = self._get_apertium()
        mockObject._get_user_locale = Mock(return_value='ca_ES')
        mockObject._get_remote_language_pairs = Mock(return_value=[['ca'], ['en'], ['ca', 'en'], ['ca|en']])
        mockObject._get_remote_language_names = Mock(return_value={'es': 'Espanyol', 'en': 'Anglès'})
        mockObject._add_missing_locale_names_in_english = Mock(return_value={'ca': 'Català', 'en': 'English'})

        mockObject._fetch_remote_language_names_and_pairs()
        self.assertEqual({'en': 'English', 'ca': 'Català'}, Apertium.g_locales_names)
        self.assertEqual(['Català -> English'], Apertium.g_language_names)
        self.assertEqual(['ca|en'], Apertium.g_language_codes)

    @patch('urllib.request.urlopen')
    def test_get_remote_language_pairs(self, mock_urlopen):
        cm = MagicMock()
        cm.getcode.return_value = 200
        cm.read.return_value = bytes('{"responseStatus": 200, "responseData": [{"targetLanguage": "cat", "sourceLanguage": "oci_aran"}]}', 'utf-8')
        cm.__enter__.return_value = cm
        mock_urlopen.return_value = cm

        apertium = self._get_apertium()
        language_pair_source, language_pair_target, locales, language_codes = apertium._get_remote_language_pairs()
        self.assertEqual(['oci_aran'], language_pair_source)
        self.assertEqual(['cat'], language_pair_target)
        self.assertEqual({'oci_aran', 'cat'}, locales)
        self.assertEqual(['oci_aran|cat'], language_codes)
        mock_urlopen.assert_called_with("https://www.apertium.org/apy/listPairs")



if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
#
#  accel_group.py - commander
#
#  Copyright (C) 2010 - Jesse van den Kieboom
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor,
#  Boston, MA 02110-1301, USA.

from gi.repository import Gtk

class Accelerator:
    def __init__(self, accelerators, arguments={}):
        if not hasattr(accelerators, '__iter__'):
            accelerators = [accelerators]

        self.accelerators = accelerators
        self.arguments = arguments

class AccelCallback:
    def __init__(self, accel, callback, data):
        self.accelerator = accel
        self.callback = callback
        self.data = data

    def activate(self, state, entry):
        self.callback(self.accelerator, self.data, state, entry)

class AccelGroup:
    def __init__(self, parent=None, name='', accelerators={}):
        self.accelerators = dict(accelerators)
        self.parent = parent
        self.name = name

    def add(self, accel, callback, data=None):
        num = len(accel.accelerators)
        mapping = self.accelerators

        for i in range(num):
            parsed = Gtk.accelerator_parse(accel.accelerators[i])

            if not Gtk.accelerator_valid(*parsed):
                return

            named = Gtk.accelerator_name(*parsed)
            inmap = named in mapping

            if i == num - 1 and inmap:
                # Last one cannot be in the map
                return
            elif inmap and isinstance(mapping[named], AccelCallback):
                # It's already mapped...
                return
            else:
                if not inmap:
                    mapping[named] = {}

                if i == num - 1:
                    mapping[named] = AccelCallback(accel, callback, data)

                mapping = mapping[named]

    def remove_real(self, accelerators, accels):
        if not accels:
            return

        parsed = Gtk.accelerator_parse(accels[0])

        if not Gtk.accelerator_valid(*parsed):
            return

        named = Gtk.accelerator_name(*parsed)

        if not named in accelerators:
            return

        if len(accels) == 1:
            del accelerators[named]
        else:
            self.remove_real(accelerators[named], accels[1:])

            if not accelerators[named]:
                del accelerators[named]

    def remove(self, accel):
        self.remove_real(self.accelerators, accel.accelerators)

    def activate(self, key, mod):
        named = Gtk.accelerator_name(key, mod)

        if not named in self.accelerators:
            return None

        accel = self.accelerators[named]

        if isinstance(accel, AccelCallback):
            return accel
        else:
            return AccelGroup(self, named, accel)

    def full_name(self):
        name = ''

        if self.parent:
            name = self.parent.full_name()

        if self.name:
            if name:
                name += ', '

            name += self.name

        return name

# ex:ts=4:et



#!/usr/bin/env python3

import os
import subprocess
import sys

libdir = sys.argv[1]
datadir = sys.argv[2]

# Packaging tools define DESTDIR and this isn't needed for them
if 'DESTDIR' not in os.environ:
    print('Compiling gsettings schemas...')
    subprocess.call(['glib-compile-schemas',
                     os.path.join(datadir, 'glib-2.0', 'schemas')])

    print('Compiling python modules...')
    subprocess.call([sys.executable, '-m', 'compileall', '-f', '-q',
                     os.path.join(libdir, 'gedit', 'plugins'),
                     os.path.join(datadir, 'gedit', 'plugins')])

    print('Compiling python modules (optimized versions) ...')
    subprocess.call([sys.executable, '-O', '-m', 'compileall', '-f', '-q',
                     os.path.join(libdir, 'gedit', 'plugins'),
                     os.path.join(datadir, 'gedit', 'plugins')])



# -*- coding: utf-8 -*-
#
#  metamodule.py
#
#  Copyright (C) 2012 - Jesse van den Kieboom
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor,
#  Boston, MA 02110-1301, USA.

class MetaModule(object):
    def __init__(self, mod):
        object.__setattr__(self, '_mod', mod)

    def __getattribute__(self, name):
        return getattr(object.__getattribute__(self, "_mod"), name)

    def __delattr__(self, name):
        delattr(object.__getattribute__(self, "_mod"), name)

    def __setattr__(self, name, value):
        setattr(object.__getattribute__(self, "_mod"), name, value)

    def __getitem__(self, item):
        return object.__getattribute__(self, "_mod")[item]

    def __setitem__(self, item, value):
        object.__getattribute__(self, "_mod")[item] = value

    def __delitem__(self, item):
        del object.__getattribute__(self, "_mod")[item]

    def __iter__(self):
        return iter(object.__getattribute__(self, "_mod"))

    def __contains__(self, item):
        return item in object.__getattribute__(self, "_mod")

    def __call__(self, *args, **kw):
        object.__getattribute__(self, "_mod").__default__(*args, **kw)

# vi:ts=4:et



# -*- coding: utf-8 -*-
#
# Copyright (c) 2019 Jordi Mas i Hernandez <jmas@softcatala.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.


import unittest
from store.session import Session

class TestSession(unittest.TestCase):

    def test_add_file(self):
        session = Session("session_A")
        session.add_file('file')
        self.assertEqual(1, len(session.files))

    def test_compare_objects(self):
        session_a = Session("session_A")
        session_b = Session("session_B")
        self.assertTrue(session_a < session_b)
        self.assertTrue(session_b > session_a)
        self.assertTrue(session_a != session_b)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
#
#  utils.py - commander
#
#  Copyright (C) 2010 - Jesse van den Kieboom
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor,
#  Boston, MA 02110-1301, USA.

import os
import types
import inspect
import sys

class Struct(dict):
    def __getattr__(self, name):
        if not name in self:
            val = super(Struct, self).__getattr__(self, name)
        else:
            val = self[name]

        return val

    def __setattr__(self, name, value):
        if not name in self:
            super(Struct, self).__setattr__(self, name, value)
        else:
            self[name] = value

    def __delattr__(self, name):
        del self[name]

def is_commander_module(mod):
    if type(mod) == types.ModuleType:
        return mod and ('__commander_module__' in mod.__dict__)
    else:
        mod = str(mod)
        return mod.endswith('.py') or (os.path.isdir(mod) and os.path.isfile(os.path.join(mod, '__init__.py')))

def getargspec(func):
    ret = inspect.getargspec(func)

    # Before 2.6 this was just a normal tuple, we don't want that
    if sys.version_info < (2, 6):
        ret = Struct({
            'args': ret[0],
            'varargs': ret[1],
            'keywords': ret[2],
            'defaults': ret[3]
        })

    return ret

# ex:ts=4:et



# -*- coding: utf-8 -*-
#
#  Copyrignt (C) 2017 Jordi Mas <jmas@softcatala.org>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor,
#  Boston, MA 02110-1301, USA.

from abc import ABCMeta, abstractmethod

class Service(metaclass=ABCMeta):

    @abstractmethod
    def translate_text(self, text, language_pair):
        pass

    @abstractmethod
    def get_language_names(self):
        pass

    @abstractmethod
    def get_language_codes(self):
        pass

    @abstractmethod
    def get_language_pair_name(self, source, target, locales_names=None):
        pass

    @abstractmethod
    def get_default_language_codes(self):
        pass

    @abstractmethod
    def has_api_key(self):
        pass

    @abstractmethod
    def get_api_hint(self):
        pass
    
    @abstractmethod    
    def set_api_key(self, key):
        pass

    @abstractmethod
    def set_server(self, server):
        pass



# -*- coding: utf-8 -*-
#
# Copyright (c) 2019 Jordi Mas i Hernandez <jmas@softcatala.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.


import unittest
from store.session import Session
from store.sessionstore import SessionStore

class TestSessionStore(unittest.TestCase):

    def test_new_two_objects(self):
        session_a = Session("session_A")
        store_a = SessionStore()
        store_a.add(session_a)

        session_b = Session("session_B")
        session_c = Session("session_C")
        store_b = SessionStore()
        store_b.add(session_b)
        store_b.add(session_c)

        self.assertEqual(1, len(store_a))
        self.assertEqual(2, len(store_b))

    def _on_session_added(self, store, session):
        self.added_called = True

    def test_add(self):
        self.added_called = False
        session = Session("session_A")
        store = SessionStore()
        store.connect_after('session-added', self._on_session_added)
        store.add(session)

        self.assertEqual(1, len(store))
        self.assertTrue(self.added_called)

    def _on_session_updated(self, store, session):
        self.updated_called = True


    def test_add_same_object_update(self):
        self.updated_called = False
        session = Session("session_A")
        store = SessionStore()
        store.connect_after('session-added', self._on_session_updated)
        store.add(session)
        session.name = 'Session B'
        store.add(session)

        self.assertEqual(1, len(store))
        self.assertEqual('Session B', store[0].name)
        self.assertTrue(self.updated_called)

    def test_add_equal_object(self):
        session_a = Session("session_A")
        session_b = Session("session_A")
        store = SessionStore()
        store.add(session_a)
        store.add(session_b)
        self.assertEqual(1, len(store))

    def test_remove(self):
        session = Session("session_A")
        store = SessionStore()
        store.add(session)
        store.remove(session)
        self.assertEqual(0, len(store))

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
#
#  Copyrignt (C) 2017 Jordi Mas <jmas@softcatala.org>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor,
#  Boston, MA 02110-1301, USA.

from gi.repository import Gio


class Settings():

    TRANSLATE_KEY_BASE = 'org.gnome.gedit.plugins.translate'
    OUTPUT_TO_DOCUMENT = 'output-to-document'
    LANGUAGE_PAIR = 'language-pair'
    SERVICE = 'service'
    API_KEY = 'api-key'
    APERTIUM_SERVER_KEY = 'apertium-server'

    def __init__(self):
        self._settings = Gio.Settings.new(self.TRANSLATE_KEY_BASE)

    def get_language_pair(self):
        return self._settings.get_string(self.LANGUAGE_PAIR)

    def get_service(self):
        return self._settings.get_uint(self.SERVICE)

    def get_apikey(self):
        return self._settings.get_string(self.API_KEY)

    def get_output_document(self):
        return self._settings.get_boolean(self.OUTPUT_TO_DOCUMENT)

    def set_language_pair(self, language_pair):
        self._settings.set_string(self.LANGUAGE_PAIR, language_pair)

    def set_service(self, service_id):
        self._settings.set_uint(self.SERVICE, service_id)

    def set_apikey(self, text):
        self._settings.set_string(self.API_KEY, text)

    def set_output_document(self, document):
        self._settings.set_boolean(self.OUTPUT_TO_DOCUMENT, document)

    def get_apertium_server(self):
        return self._settings.get_string(self.APERTIUM_SERVER_KEY)



# -*- coding: utf-8 -*-
#
#  align.py - align commander module
#
#  Copyright (C) 2010 - Jesse van den Kieboom
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor,
#  Boston, MA 02110-1301, USA.

import commander.commands as commands
import commander.commands.completion
import commander.commands.result
import commander.commands.exceptions

from functools import reduce
import re

__commander_module__ = True

def _get_groups(m, group, add_ws_group):
    if len(m.groups()) <= group - 1:
        gidx = 0
    else:
        gidx = group

    if len(m.groups()) <= add_ws_group - 1:
        wsidx = 0
    else:
        wsidx = add_ws_group

    # Whitespace group must be contained in align group
    if m.start(wsidx) < m.start(gidx) or m.end(wsidx) > m.end(gidx):
        wsidx = gidx

    return (gidx, wsidx)

class Line:
    def __init__(self, line, reg, tabwidth):
        self.tabwidth = tabwidth
        self.line = line

        # All the separators
        self.matches = list(reg.finditer(line))

        # @newline initially contains the first column

        if not self.matches:
            # No separator found
            self.newline = str(line)
        else:
            # Up to first separator
            self.newline = line[0:self.matches[0].start(0)]

    def matches_len(self):
        return len(self.matches)

    def new_len(self, extra=''):
        return len((self.newline + extra).expandtabs(self.tabwidth))

    def match(self, idx):
        if idx >= self.matches_len():
            return None

        return self.matches[idx]

    def append(self, idx, num, group, add_ws_group):
        m = self.match(idx)

        if m == None:
            return

        gidx, wsidx = _get_groups(m, group, add_ws_group)

        # Append leading match
        self.newline += self.line[m.start(0):m.start(gidx)]

        # Now align by replacing wsidx with spaces
        prefix = self.line[m.start(gidx):m.start(wsidx)]
        suffix = self.line[m.end(wsidx):m.end(gidx)]
        sp = ''

        while True:
            bridge = prefix + sp + suffix

            if self.new_len(bridge) < num:
                sp += ' '
            else:
                break

        self.newline += bridge

        # Then append the rest of the match
        mnext = self.match(idx + 1)

        if mnext == None:
            endidx = None
        else:
            endidx = mnext.start(0)

        self.newline += self.line[m.end(gidx):endidx]

    def __str__(self):
        return self.newline

def _find_max_align(lines, idx, group, add_ws_group):
    num = 0

    # We will align on 'group', by adding spaces to 'add_ws_group'
    for line in lines:
        m = line.match(idx)

        if m != None:
            gidx, wsidx = _get_groups(m, group, add_ws_group)

            # until the start
            extra = line.line[m.start(0):m.start(wsidx)] + line.line[m.end(wsidx):m.end(gidx)]

            # Measure where to align it
            l = line.new_len(extra)
        else:
            l = line.new_len()

        if l > num:
            num = l

    return num

def _regex(view, reg, group, additional_ws, add_ws_group, flags=0):
    buf = view.get_buffer()

    # Get the selection of lines to align columns on
    bounds = buf.get_selection_bounds()

    if not bounds:
        start = buf.get_iter_at_mark(buf.get_insert())
        start.set_line_offset(0)

        end = start.copy()

        if not end.ends_line():
            end.forward_to_line_end()

        bounds = (start, end)

    if not bounds[0].equal(bounds[1]) and bounds[1].starts_line():
        bounds[1].backward_line()

        if not bounds[1].ends_line():
            bounds[1].forward_to_line_end()

    # Get the regular expression from the user
    if reg == None:
        reg, words, modifier = (yield commander.commands.result.Prompt('Regex:'))

    # Compile the regular expression
    try:
        reg = re.compile(reg, flags)
    except Exception as e:
        raise commander.commands.exceptions.Execute('Failed to compile regular expression: %s' % (e,))

    # Query the user to provide a regex group number to align on
    if group == None:
        group, words, modifier = (yield commander.commands.result.Prompt('Group (1):'))

    try:
        group = int(group)
    except:
        group = 1

    # Query the user for additional whitespace to insert for separating items
    if additional_ws == None:
        additional_ws, words, modifier = (yield commander.commands.result.Prompt('Additional whitespace (0):'))

    try:
        additional_ws = int(additional_ws)
    except:
        additional_ws = 0

    # Query the user for the regex group number on which to add the
    # whitespace
    if add_ws_group == None:
        add_ws_group, words, modifier = (yield commander.commands.result.Prompt('Whitespace group (1):'))

    try:
        add_ws_group = int(add_ws_group)
    except:
        add_ws_group = -1

    # By default, add the whitespace on the group on which the columns are
    # aligned
    if add_ws_group < 0:
        add_ws_group = group

    start, end = bounds

    if not start.starts_line():
        start.set_line_offset(0)

    if not end.ends_line():
        end.forward_to_line_end()

    lines = start.get_text(end).splitlines()
    newlines = []
    num = 0
    tabwidth = view.get_tab_width()

    # Construct Line objects for all the lines
    newlines = [Line(line, reg, tabwidth) for line in lines]

    # Calculate maximum number of matches (i.e. columns)
    num = reduce(lambda x, y: max(x, y.matches_len()), newlines, 0)

    for i in range(num):
        al = _find_max_align(newlines, i, group, add_ws_group)

        for line in newlines:
            line.append(i, al + additional_ws, group, add_ws_group)

    # Replace lines
    aligned = str.join('\n', [x.newline for x in newlines])

    buf.begin_user_action()
    buf.delete(bounds[0], bounds[1])

    m = buf.create_mark(None, bounds[0], True)

    buf.insert(bounds[1], aligned)
    buf.select_range(buf.get_iter_at_mark(m), bounds[1])
    buf.delete_mark(m)

    buf.end_user_action()

    yield commander.commands.result.DONE

def __default__(view, reg='\s+', align_group=1, padding=1, padding_group=-1):
    """Align selected in columns using a regular expression: align.regex [&lt;regex&gt;=<i>\s+</i>] [&lt;align-group&gt;] [&lt;padding&gt;] [&lt;padding-group&gt;=<i>&lt;align-group&gt;</i>]

Align the selected text in columns separated by the specified regular expression.

The optional &lt;align-group&gt; argument specifies on which group in the regular expression
the text should be aligned and defaults to 1 (or 0 in the case that there is
no explicit group specified). The &lt;align-group&gt; will be <b>replaced</b>
with whitespace to align the columns. The optional &lt;padding&gt; argument can
be used to add additional whitespace to the column separation. The last
optional argument (&lt;padding-group&gt;) can be used to specify a separate
group (which must be contained in &lt;align-group&gt;) which to replace with
whitespace.

The regular expression will be matched in case-sensitive mode"""

    yield _regex(view, reg, align_group, padding, padding_group)

def i(view, reg='\s+', align_group=1, padding=1, padding_group=-1):
    """Align selected in columns using a regular expression: align.regex [&lt;regex&gt;=<i>\s+</i>] [&lt;align-group&gt;] [&lt;padding&gt;] [&lt;padding-group&gt;=<i>&lt;align-group&gt;</i>]

Align the selected text in columns separated by the specified regular expression.

The optional &lt;align-group&gt; argument specifies on which group in the regular expression
the text should be aligned and defaults to 1 (or 0 in the case that there is
no explicit group specified). The &lt;align-group&gt; will be <b>replaced</b>
with whitespace to align the columns. The optional &lt;padding&gt; argument can
be used to add additional whitespace to the column separation. The last
optional argument (&lt;padding-group&gt;) can be used to specify a separate
group (which must be contained in &lt;align-group&gt;) which to replace with
whitespace.

The regular expression will be matched in case-insensitive mode"""

    yield _regex(view, reg, align_group, padding, padding_group, re.IGNORECASE)

# ex:ts=4:et



# -*- coding: utf-8 -*-
#
#  Copyrignt (C) 2017 Jordi Mas <jmas@softcatala.org>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor,
#  Boston, MA 02110-1301, USA.


from gi.repository import GLib, Gtk

class TranslateView(Gtk.ScrolledWindow):

    def __init__(self, namespace = {}):
        Gtk.ScrolledWindow.__init__(self)

        self.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
        self.set_shadow_type(Gtk.ShadowType.NONE)
        self.view = Gtk.TextView()
        self.view.set_editable(False)
        self.view.set_wrap_mode(Gtk.WrapMode.WORD_CHAR)
        self.add(self.view)
        self.view.show()

    def do_grab_focus(self):
        self.view.grab_focus()

    def stop(self):
        self.namespace = None

    def scroll_to_end(self):
        i = self.view.get_buffer().get_end_iter()
        self.view.scroll_to_iter(i, 0.0, False, 0.5, 0.5)
        return False

    def write(self, text):
        text = text + "\n"
        buf = self.view.get_buffer()
        buf.insert(buf.get_end_iter(), text)
        GLib.idle_add(self.scroll_to_end)

    def destroy(self):
        pass

